#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCH_1 1
#define VENDOR_RQ_GET_SWITCH_2 8
#define VENDOR_RQ_GET_SWITCH_3 3
#define VENDOR_RQ_GET_SWITCH_4 4
#define VENDOR_RQ_GET_SWITCH_5 5
#define VENDOR_RQ_GET_SWITCH_6 6
//
#define VENDOR_RQ_GET_SWITCH_7 12
//
#define VENDOR_RQ_GET_LIGHT_1  2
#define VENDOR_RQ_GET_LIGHT_2  7
#define VENDOR_RQ_GET_LIGHT_3  9
#define VENDOR_RQ_GET_LIGHT_4  10
#define VENDOR_RQ_GET_LIGHT_5  11


/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

uint16_t A;


uint8_t in_c(uint8_t pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return (PINC >>pin) & 1;
} 

void out_c(uint8_t pin, uint8_t val)
{
    // write logic level to port-C's pin
    DDRC |= 1<<pin;
    if (val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}


uint8_t in_b(uint8_t pin)
{
    // read logic level of port-D's pin
    DDRB &= ~(1<<pin);
    return (PINB >>pin) & 1;
} 

void out_b(uint8_t pin, uint8_t val)
{
    // write logic level to port-D's pin
    DDRB |= 1<<pin;
    if (val)
        PORTB |= 1<<pin;
    else
        PORTB &= ~(1<<pin);
}

uint8_t in_d(uint8_t pin)
{
    // read logic level of port-D's pin
    DDRD &= ~(1<<pin);
    return (PIND >>pin) & 1;
} 

void out_d(uint8_t pin, uint8_t val)
{
    // write logic level to port-D's pin
    DDRD |= 1<<pin;
    if (val)
        PORTD |= 1<<pin;
    else
        PORTD &= ~(1<<pin);
}

uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
    ADMUX |= channel;   // ตั้ง MUX ให้ชี้ไปยัง channel
    ADCSRA = 0b11000110;
    //         ||   -+-
    //         ||    |
    //         ||    +-> ปรับคล็อกของ ADC ให้เป็น 1/64 ของคล็อกระบบ
    //         |+->เริ่มแปลงค่าแอนะล็อกเป็นดิจิทัล (บิต ADSC)
    //         |
    //         +-> Enable ADC (บิต ADEN)

    while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
        ;
    return ADCL+ ADCH*256;  // ผลลัพธ์ถูกเก็บอยู่ในรีจีสเตอร์ ADCL และ ADCH
}


/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t dataBuffer[1];  /* must stay valid when usbFunctionSetup returns */
    static uint16_t dataBuffer2[1];

    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_c(led_no, led_val);
        return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH_1)
    {
        dataBuffer[0] = !in_b(1); //อ่านจาก port 0 2
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH_2)
    {
        dataBuffer[0] = !in_b(0); //อ่านจาก port 1 1
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH_3)
    {
        dataBuffer[0] = !in_b(3); //อ่านจาก port 2 4
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH_4)
    {
        dataBuffer[0] = !in_b(2); //อ่านจาก port 3 3
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH_5)
    {
        dataBuffer[0] = !in_b(5); //อ่านจาก port 4 6
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH_6)
    {
        dataBuffer[0] = !in_b(4); //อ่านจาก port 5 5
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH_7)
    {
        dataBuffer[0] = !in_d(6); //อ่านจาก port 5 5
        usbMsgPtr = dataBuffer;
        return 1;
    }




	else if (rq->bRequest == VENDOR_RQ_GET_LIGHT_1)
	{
		A = read_adc(4);        
        dataBuffer[0] = A;
        dataBuffer[1] = A>>8;
        usbMsgPtr = dataBuffer;
        return 2;
	}
	else if (rq->bRequest == VENDOR_RQ_GET_LIGHT_2)
	{
		A = read_adc(2);
		dataBuffer[0] = A;
		dataBuffer[1] =	A>>8;
		usbMsgPtr = dataBuffer;
		return 2;
	}
	else if (rq->bRequest == VENDOR_RQ_GET_LIGHT_3)
	{
		A = read_adc(0);
		dataBuffer[0] = A;
		dataBuffer[1] =	A>>8;
		usbMsgPtr = dataBuffer;
		return 2;
	}
	else if (rq->bRequest == VENDOR_RQ_GET_LIGHT_4)
	{
		A = read_adc(1);
		dataBuffer[0] = A;
		dataBuffer[1] =	A>>8;
		usbMsgPtr = dataBuffer;
		return 2;
	}
	else if (rq->bRequest == VENDOR_RQ_GET_LIGHT_5)
	{
		A = read_adc(3);
		dataBuffer[0] = A;
		dataBuffer[1] =	A>>8;
		usbMsgPtr = dataBuffer;
		return 2;
	}



    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRC  = 0b00000000; // PC0..PC2 = Output, PC3..PC5 = Input
	DDRB  = 0b00000000;
	DDRD  = 0b00000000;

	PORTC = 0b00111111;                 // Enable pull-up at PC3
	PORTB = 0b00111111;
	PORTD = 0b00000010;
  

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
		 usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
